/*
Brute-force hash cracker.  Checks SHA-1 hash of various strings until
it hits the one it's shooting for.

Orion Sky Lawlor, olawlor@acm.org, 2006/02/27 (public domain)
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sha1.h"
#include "sha1.cpp"

/* Hash code of the password we're searching for */
SHA1_hash_t target;

/* If 1, print out test strings as they're checked */
int verbose=0;

/* Check if this C string has the same hash as the target */
void check(const char *str,int len=-1) {
	if (len<0) len=strlen(str);
	SHA1_hash_t h=SHA1_hash(str,len);
	if (verbose)
		printf("Checking against %d-byte string '%s' -> %s\n",
			strlen(str),str,SHA1_hex_from_hash(h).c_str());
	if (h==target) {
		printf("Target hash matches '%s'!\n",str);
		exit(0);
	}
}

int main(int argc,char *argv[]) {
	const char *targetString=argv[1];
	if (targetString==NULL) {
		printf("Usage: brute <target SHA-1 hash>\n"); 
		exit(1);
	}
	target=SHA1_hash_from_hex(targetString);
	printf("Searching target space...\n");
	char str[32]; memset(str,0,32); /* repeatedly nul-terminated */
	
	printf("Checking all single-char passwords\n");
	for (str[0]=0;str[0]!=(char)0xff;str[0]++) check(str,1);
	
	printf("Checking all two-byte passwords\n");
	for (str[0]=0;str[0]!=(char)0xff;str[0]++) 
	for (str[1]=0;str[1]!=(char)0xff;str[1]++) check(str,2);
	
	printf("Checking all 4-digit passwords\n");
	for (str[0]='0';str[0]<='9';str[0]++) 
	for (str[1]='0';str[1]<='9';str[1]++) 
	for (str[2]='0';str[2]<='9';str[2]++) 
	for (str[3]='0';str[3]<='9';str[3]++) check(str);
	str[3]=0; /* password getting shorter--cut off */
	
	printf("Checking all three-char passwords\n");
	for (str[0]=' ';str[0]!='~';str[0]++)
	for (str[1]=' ';str[1]!='~';str[1]++)
	for (str[2]=' ';str[2]!='~';str[2]++) check(str);
	
	printf("Checking all 8-digit 'skip' passwords\n");
	for (int s1=0;s1<9;s1++)
	for (int s2=0;s2<9;s2++) {
		for (int i=0;i<8;i++) 
			str[i]='0'+i+(i>=s1)+(i>=s2); 
		check(str);
	}
	
	printf("Checking all 10-digit 'foofoo' passwords\n");
	for (int s=0;s<(1<<10);s++) {
		for (int bit=0;bit<10;bit++)
			str[bit]=(s&(1<<bit))?'f':'o';
		check(str);
	}
	
	printf("Couldn't find matching hash...\n");
	return 0;
}
